-- Copyright (c) 2011 Cloudera, Inc. All rights reserved.
--
-- Version 12 of the CMF schema. Migration code to be applied on top of the
-- previous version.
--

update SCHEMA_VERSION set VERSION=12;
--
-- Modifications for configuration revisions.

--
-- We'll need this sequence to auto-increment IDs when creating base
-- revisions. A 'serial' column type would be ideal but deleting the
-- associated sequence doesn't work in certain version of Postgres. See
-- http://archives.postgresql.org/pgsql-bugs/2006-03/msg00159.php for details.
create sequence CONFIG_REVISIONS_REVISION_ID_SEQ;

--
-- Each service with configs must be tied to a configuration revision. When the
-- configs are modified, a new revision is created and all non-inherited
-- configs are added to it.
create table CONFIG_REVISIONS (
    REVISION_ID int8 not null default nextval('CONFIG_REVISIONS_REVISION_ID_SEQ'),
    SERVICE_ID int8 not null,
    USER_ID int8,
    CREATED_INSTANT int8 not null,
    MESSAGE varchar(1048576) not null,
    primary key (REVISION_ID)
);

alter table SERVICES
    add column CONFIG_REVISION_ID int8;

alter table CONFIGS
    add column REVISION_ID int8;

--
-- Insert a base revision for each service with existing configs. Note that
-- automatic incrementing of the revision ID was enabled when the table was
-- created (by using a 'nextval' default based on a sequence); it is disabled
-- following this insertion.
insert into CONFIG_REVISIONS (SERVICE_ID, CREATED_INSTANT, MESSAGE)
    select SERVICE_ID, extract(epoch FROM now()) * 1000, 'Base revision'
    from SERVICES s
    where exists
        (select null
         from CONFIGS c
         where c.SERVICE_ID = s.SERVICE_ID);
alter table CONFIG_REVISIONS
    alter column REVISION_ID drop default;

drop sequence CONFIG_REVISIONS_REVISION_ID_SEQ;

--
-- Link each service to its current config revision.
--
-- Note that UPDATE doesn't allow aliases in older versions of Postgres, so
-- they aren't used here.
update SERVICES
    set CONFIG_REVISION_ID=
        (select REVISION_ID
         from CONFIG_REVISIONS r
         where SERVICES.SERVICE_ID=r.SERVICE_ID);

--
-- Link each config to the current config revision of its service.
update CONFIGS
    set REVISION_ID=
        (select REVISION_ID
         from CONFIG_REVISIONS r
         where CONFIGS.SERVICE_ID=r.SERVICE_ID);

--
-- All data insertion is done. Set up indices and constraints.
--
alter table CONFIGS
    alter column REVISION_ID set not null;

create index IDX_SERVICE_CONFIG_REVISION on SERVICES (CONFIG_REVISION_ID);

create index IDX_CONFIG_REVISION_SERVICE on CONFIG_REVISIONS (SERVICE_ID);

create index IDX_CONFIG_CONFIG_REVISION on CONFIGS (REVISION_ID);

alter table SERVICES
    add constraint FK_SERVICE_CONFIG_REVISION
    foreign key (CONFIG_REVISION_ID)
    references CONFIG_REVISIONS (REVISION_ID);

alter table CONFIG_REVISIONS
    add constraint FK_CONFIG_REVISION_SERVICE
    foreign key (SERVICE_ID)
    references SERVICES (SERVICE_ID);

alter table CONFIGS
    add constraint FK_CONFIG_CONFIG_REVISION
    foreign key (REVISION_ID)
    references CONFIG_REVISIONS (REVISION_ID);
